(function($) {

    $.gamesRegistry('register', 'word', new (function() {

	var options = {
	    view_width: Infinity,
	    view_height: Infinity,
	    view_background_img: '',
	    draggable: []
	};

	this.functionToMake = new Array();

	this.inRightPlaces = 0;
	this.boardNo = 0;

	this.restart = function()
	{
	    this.boardNo = 0;
	    this.inRightPlaces = 0;

	    $('#playground').find('#canvas').remove();
	    $('#playground').find('.img').remove();
	    $('#playground').find('.place_for_drop').remove();
	    $('#playground').find('.for_drag').remove();
	};

	this.congrats = function() {
	    $('.task-content').addClass('hide');

	    setTimeout(
		function() {
		    $('#congratulation-panel').fadeIn('slow');

		    if ($('.button.sound.off').length === 1) {
			return;
		    }

		    if (this.snd_instance !== undefined) {
			this.snd_instance.stop();
		    }

		    this.snd_instance = createjs.Sound.play(
			options.sound_congrats,
			createjs.Sound.INTERRUPT_NONE,
			0, //delay
			0, //offset
			0, //loop
			0 //pan
			);

		    this.snd_instance.setVolume(3);
		},
		800
		);
	};

	this.getBoard = function()
	{
	    return this.options.games[this.boardNo];
	};

	this.makeDraggable = function(obj)
	{
	    obj.draggable({
		revert: true,
		start: function() {
		    $(this).addClass('drag-now');
		},
		stop: function() {
		    $(this).removeClass('drag-now');
		}
	    });
	};

	this.playSound = function(ok)
	{
	    var sound;

	    if (ok) {
		var soundId = Math.floor(Math.random() * options.ok_sounds.length);
		sound = options.ok_sounds[soundId];
	    } else {
		var soundId = Math.floor(Math.random() * options.fail_sounds.length);
		sound = options.fail_sounds[soundId];
	    }

	    if ($('.sound.button.off').length === 1) {
		return;
	    }

	    if (this.snd_instance !== undefined) {
		this.snd_instance.stop();
	    }

	    if (snd_instance !== undefined) {
		snd_instance.pause();
	    }

	    if (this.characterSound) {
		this.characterSound.stop();
	    }

	    this.characterSound = createjs.Sound.play(sound, createjs.Sound.INTERUPT_LATE);
	    this.characterSound.addEventListener("complete", function() {
		if (snd_instance !== undefined) {
		    snd_instance.resume();
		}
	    });
	};

	this.createObj = function(data) {

	    var obj = $('<div class="object"></div>');

	    obj.css('background-image', 'url("' + preload.getResult(data.image).src + '")');

	    obj.width(100 * data.width / options.view_width + '%');
	    obj.height(100 * data.height / options.view_height + '%');
	    obj.css({
		left: 100 * data.x / options.view_width + '%',
		top: 100 * data.y / options.view_height + '%',
	    });

	    if (data.visible !== undefined && data.visible === false) {
		obj.hide();
		obj.addClass('object-hidden');
		obj.attr('id', data.image);
	    }

	    return obj;
	};

	this.showNextBoard = function()
	{
	    var that = this;
	    var time = 500;

	    that.boardNo++;

	    if (that.boardNo >= that.options.games.length) {
		that.showCongrats();
	    } else {
		setTimeout(function(){
		    $('#playground').find('#canvas').fadeOut(time);
		    $('#playground').find('.img').fadeOut(time);
		    $('#playground').find('.place_for_drop').fadeOut(time);
		    $('#playground').find('.for_drag').fadeOut(time);

		    setTimeout(function() {
			$('#playground').find('#canvas').remove();
			$('#playground').find('.img').remove();
			$('#playground').find('.place_for_drop').remove();
			$('#playground').find('.for_drag').remove();
			that.generateAndShowBoard();
		    }, time);
		}, 1000);
	    }
	};
	
	this.showPrevBoard = function(){
	    var that = this;
	    var time = 500;

	    that.boardNo--;
	    
	    if (that.boardNo < 0) {
		goToStartPage();
	    } else {
		setTimeout(function(){
		    $('#playground').find('#canvas').fadeOut(time);
		    $('#playground').find('.img').fadeOut(time);
		    $('#playground').find('.place_for_drop').fadeOut(time);
		    $('#playground').find('.for_drag').fadeOut(time);

		    setTimeout(function() {
			$('#playground').find('#canvas').remove();
			$('#playground').find('.img').remove();
			$('#playground').find('.place_for_drop').remove();
			$('#playground').find('.for_drag').remove();
			that.generateAndShowBoard();
		    }, time);
		}, 1000);
	    }
	};

	this.showCongrats = function()
	{
	    $('#congratulation-panel').fadeIn();
	};

	this.makeDroppable = function(placeForDrop)
	{
	    var that = this;
	    placeForDrop.droppable({
		tolerance: 'pointer',
		drop: function(event, ui) {
		    var elem = $(this);
		    if ($(this).hasClass(ui.draggable.attr('id'))) {
			ui.draggable.draggable({revert: false});

			ui.draggable.animate({
			    'top': $(this).css('top'),
			    'left': $(this).css('left')
			}, 200);
			
			setTimeout(function(){
			    ui.draggable.remove();
			    elem.css('opacity', 1);
			}, 200);
			
			that.functionToMake = that.functionToMake.concat([true]);

			that.inRightPlaces++;
			if (that.inRightPlaces === that.getBoard().elements.length) {
			    that.playSound(true);
			    $('#IMG_postac_ok').fadeIn();
			    setTimeout(function(){
				$('#IMG_postac_ok').fadeOut();
			    }, 1500);
			    that.showNextBoard();
			}

		    } else {
			that.functionToMake = that.functionToMake.concat([false]);
		    }

		    setTimeout(function() {
			if (that.functionToMake.length === 0) {
			    return;
			}

			var isOneTrue = false;
			for (var i = 0; i < that.functionToMake.length; i++) {
			    if (that.functionToMake[i]) {
				isOneTrue = true;
				break;
			    }
			}

			that.functionToMake = new Array();

			if (isOneTrue) {
			    return;
			}

			that.makeRed();
		    }, 10);
		}
	    });
	}

	this.generateAndShowBoard = function()
	{
	    var that = this;
	    this.inRightPlaces = 0;

	    var elements = this.getBoard();

	    var canvas = $('<div id="canvas">');
	    canvas.css({
		position: 'absolute',
		width: elements.canvas.width + '%',
		height: elements.canvas.height + '%',
		top: elements.canvas.top + '%',
		left: elements.canvas.left + '%',
		backgroundColor: 'white'
	    });
	    canvas.hide();
	    $('#playground').append(canvas);
	    this.makeDroppable(canvas);

	    var half = $('<div class="img">');
	    half.css({
		width: elements.half.width + '%',
		height: elements.half.height + '%',
		top: elements.half.top + '%',
		left: elements.half.left + '%',
		backgroundImage: "url('" + preload.getResult(elements.half.image).src + "')"
	    });
	    half.hide();
	    $('#playground').append(half);

	    $.each(elements.elements, function() {
		var element = this;

		var placeForDrop = $('<div class="place_for_drop ' + this.image + '">');
		placeForDrop.css({
		    width: this.finall_position.width + '%',
		    height: this.finall_position.height + '%',
		    top: this.finall_position.top + '%',
		    left: this.finall_position.left + '%',
		    backgroundImage: "url('" + preload.getResult(this.image).src + "')"
		});
		placeForDrop.hide();
		$('#playground').append(placeForDrop);
		that.makeDroppable(placeForDrop);

		var forDrag = $('<div class="for_drag" id="' + this.image + '">');
		forDrag.css({
		    width: this.position.width + '%',
		    height: this.position.height + '%',
		    top: this.position.top + '%',
		    left: this.position.left + '%',
		    backgroundImage: "url('" + preload.getResult(this.image).src + "')"
		});
		forDrag.hide();
		$('#playground').append(forDrag);
		that.makeDraggable(forDrag);
	    });

	    $('#playground').find('#canvas').fadeIn();
	    $('#playground').find('.img').fadeIn();
	    $('#playground').find('.place_for_drop').fadeIn();
	    $('#playground').find('.for_drag').fadeIn();
	};

	this.makeRed = function() {
	    $('#canvas')
		.css('background-color', 'rgb(255, 179, 179)')
		.animate({
		backgroundColor: 'white'
	    }, 300);
	};

	this.init = function(data, endCallback) {
	    var that = this;

	    $.extend(options, data);
	    this.options = options;

	    // Set background
	    var background = preload.getResult(options.view_background_img).src;
	    $('#playground').css('background-image', 'url("' + background + '")');

	    $.each(options.objects, function() {
		$('#playground').append(that.createObj(this));
	    });

	    this.generateAndShowBoard();

	    $('.repeat_game').unbind('click').bind('click', function() {
		$('#congratulation-panel').fadeOut(500);

		setTimeout(function() {
		    that.boardNo = 1;
		    $('.task-content').removeClass('hide');
		    goToStartPage();
		}, 800);
	    });
	    $('.refresh.button').unbind('click').bind('click', function() {

		if (that.characterSound) {
		    that.characterSound.stop();
		}

		restartPrompt(function() {
		    $('#congratulation-panel').fadeOut(500);

		    setTimeout(function() {
			that.boardNo = 1;
			$('.task-content').removeClass('hide');
			goToStartPage();
		    }, 800);
		});
	    });


	    $('.sound.button').bind('click', function() {
		if ($('.sound.button.off').length === 1) {
		    that.characterSound.stop();
		}
	    });

	    return this;
	};
    })());

})(jQuery);

